@@ -19,15 +19,13 @@ import os
 import re
 import sys
 import time
-import traceback
 
 import jinja2
 import jinja2.meta
-import jsonschema
-from oslo_config import cfg
 import requests
 from requests.packages import urllib3
 
+from rally.common import cfg
 from rally.common import logging
 from rally.common import objects
 from rally.common import opts
@@ -35,7 +33,6 @@ from rally.common.plugin import discover
 from rally.common import utils
 from rally.common import version as rally_version
 from rally import consts
-from rally.deployment import engine as deploy_engine
 from rally import exceptions
 from rally.task import engine
 from rally.task import exporter as texporter
@@ -73,45 +70,37 @@ class _Deployment(APIGroup):
 
         # NOTE(andreykurilin): the following transformation is a preparatory
         #   step for further refactoring (it will be done soon).
-        print_warning = True
-        if "type" not in config:
-            # it looks like a new format! wow!
-            config = {"type": "ExistingCloud",
-                      "creds": config}
-            print_warning = False
+        print_warning = False
+
+        extras = {}
+        if "type" in config:
+            if config["type"] != "ExistingCloud":
+                raise exceptions.RallyException(
+                    "You are using deployment type which doesn't exist. Please"
+                    " check the latest documentation and fix deployment "
+                    "config.")
+
+            config = config["creds"]
+            extras = config.get("extra", {})
+            print_warning = True
 
         try:
-            deployment = objects.Deployment(name=name, config=config)
-        except exceptions.DeploymentNameExists:
+            deployment = objects.Deployment(name=name, config=config,
+                                            extras=extras)
+        except exceptions.DBRecordExists:
             if logging.is_debug():
                 LOG.exception("Deployment with such name exists")
             raise
 
-        deployer = deploy_engine.Engine.get_engine(
-            deployment["config"]["type"], deployment)
-        try:
-            deployer.validate()
-        except jsonschema.ValidationError:
-            LOG.error("Deployment %s: Schema validation error." %
-                      deployment["uuid"])
-            deployment.update_status(consts.DeployStatus.DEPLOY_FAILED)
-            raise
-
-        if print_warning and config.get("type", "") == "ExistingCloud":
-            # credentials are stored in the list, but it contains one item.
-            new_conf = dict(
-                (name, cred[0])
-                for name, cred in deployer._get_creds(config).items())
+        if print_warning:
+            new_conf = deployment.env_obj.spec
             LOG.warning(
                 "The used config schema is deprecated since Rally 0.10.0. "
                 "The new one is much simpler, try it now:\n%s"
                 % json.dumps(new_conf, indent=4)
             )
 
-        with deployer:
-            credentials = deployer.make_deploy()
-            deployment.update_credentials(credentials)
-            return deployment
+        return deployment
 
     def create(self, config, name):
         return self._create(config, name).to_dict()
@@ -121,26 +110,11 @@ class _Deployment(APIGroup):
 
         :param deployment: UUID or name of the deployment
         """
-        # TODO(akscram): We have to be sure that there are no running
-        #                tasks for this deployment.
-        # TODO(akscram): Check that the deployment have got a status that
-        #                is equal to "*->finished" or "deploy->inconsistent".
-        deployment = objects.Deployment.get(deployment)
-        try:
-            deployer = deploy_engine.Engine.get_engine(
-                deployment["config"]["type"], deployment)
-            with deployer:
-                deployer.make_cleanup()
-        except exceptions.PluginNotFound:
-            LOG.info("Deployment %s will be deleted despite exception"
-                     % deployment["uuid"])
 
-        for verifier in self.api.verifier.list():
-            self.api.verifier.delete(verifier_id=verifier["name"],
-                                     deployment_id=deployment["name"],
-                                     force=True)
+        deploy = objects.Deployment.get(deployment)
 
-        deployment.delete()
+        deploy.env_obj.destroy(skip_cleanup=True)
+        deploy.env_obj.delete()
 
     def recreate(self, deployment, config=None):
         """Performs a cleanup and then makes a deployment again.
@@ -149,47 +123,8 @@ class _Deployment(APIGroup):
         :param config: an optional dict with deployment config to update before
                        redeploy
         """
-        deployment = objects.Deployment.get(deployment)
-        deployer = deploy_engine.Engine.get_engine(
-            deployment["config"]["type"], deployment)
-
-        print_warning = True
-        if config and "type" not in config:
-            # it looks like a new format! wow!
-            config = {"type": "ExistingCloud",
-                      "creds": config}
-            print_warning = False
-
-        if config:
-            if deployment["config"]["type"] != config["type"]:
-                raise exceptions.RallyException(
-                    "Can't change deployment type.")
-            try:
-                deployer.validate(config)
-            except jsonschema.ValidationError:
-                LOG.error("Config schema validation error.")
-                raise
-
-        if (config and print_warning and
-                config.get("type", "") == "ExistingCloud"):
-            # credentials are stored in the list, but it contains one item.
-            new_conf = dict(
-                (name, cred[0])
-                for name, cred in deployer._get_creds(config).items())
-            LOG.warning(
-                "The used config schema is deprecated since Rally 0.10.0. "
-                "The new one is much simpler, try it now:\n%s"
-                % json.dumps(new_conf, indent=4)
-            )
-
-        with deployer:
-            deployer.make_cleanup()
-
-            if config:
-                deployment.update_config(config)
-
-            credentials = deployer.make_deploy()
-            deployment.update_credentials(credentials)
+        raise exceptions.RallyException("Sorry, but recreate method of "
+                                        "deployments is temporary disabled.")
 
     def _get(self, deployment):
         """Get the deployment.
@@ -200,24 +135,7 @@ class _Deployment(APIGroup):
         return objects.Deployment.get(deployment)
 
     def get(self, deployment):
-        deployment = self._get(deployment).to_dict()
-        if deployment["config"].get("type", "") == "ExistingCloud":
-            deployment_creds = {}
-            if "creds" not in deployment["config"]:
-                extra = deployment["config"].pop("extra", None)
-                deployment["config"] = {
-                    "type": deployment["config"].pop("type"),
-                    "creds": {"openstack": deployment["config"]}
-                }
-                if extra is not None:
-                    deployment["config"]["extra"] = extra
-            for platform, creds in deployment["config"]["creds"].items():
-                if isinstance(creds, dict):
-                    deployment_creds[platform] = creds
-                else:
-                    deployment_creds[platform] = creds[0]
-            deployment["config"] = deployment_creds
-        return deployment
+        return self._get(deployment).to_dict()
 
     def service_list(self, deployment):
         """Get the services list.
@@ -243,39 +161,56 @@ class _Deployment(APIGroup):
         :param deployment: UUID of deployment
         :returns: Service list
         """
-        result = {}
-        all_credentials = self._get(deployment).get_all_credentials()
-        for platform in all_credentials:
-            result[platform] = []
-            for credential in all_credentials[platform]:
-                no_error = True
-                result[platform].append({"services": []})
-                active_user = None
-                if credential["admin"]:
-                    active_user = credential["admin"]
-                    try:
-                        credential["admin"].verify_connection()
-                    except Exception as e:
-                        no_error = False
-                        result[platform][-1]["admin_error"] = {
-                            "etype": e.__class__.__name__,
-                            "msg": str(e),
-                            "trace": traceback.format_exc()}
-                for user in credential["users"]:
-                    try:
-                        user.verify_connection()
-                    except Exception as e:
-                        no_error = False
-                        result[platform][-1]["user_error"] = {
-                            "etype": e.__class__.__name__,
-                            "msg": str(e),
-                            "trace": traceback.format_exc()}
-                        break
+        env = self._get(deployment).env_obj
 
-                if no_error:
-                    active_user = active_user or credential["users"][0]
-                    services = active_user.list_services()
-                    result[platform][-1]["services"] = services
+        result = {}
+
+        for p, res in env.check_health().items():
+            name = "openstack" if p == "existing@openstack" else p
+            if not res["available"]:
+                # NOTE(andreykurilin): the old behavior supports 2 keys
+                #   for storing errors: admin_error and user_error.
+                #   Since admin/users is not mandatory thing for new design
+                #   of Platforms, let's put platform error to "admin_error"
+                key = "admin_error"
+                if name == "openstack":
+                    if res["message"].startswith("Bad user creds"):
+                        key = "user_error"
+
+                # NOTE(andreykurilin): the last not null line in traceback
+                #   includes Exception cls with a message. By parsing it, we
+                #   can get etype.
+                trace = res["traceback"].split("\n")
+                last_line = [l for l in trace if l][-1]
+                etype, _msg = last_line.split(":", 1)
+                result[name] = [
+                    {
+                        key: {
+                            "etype": etype,
+                            "msg": res["message"],
+                            "trace": res["traceback"]
+                        },
+                        "services": []
+                    }
+                ]
+            else:
+                if name == "openstack":
+                    services = env.get_info()[p]["info"]["services"]
+                    # backward compatibility
+                    for s in services:
+                        if "name" not in s:
+                            s["name"] = "__unknown__"
+
+                    result[name] = [{"services": services}]
+                else:
+                    # NOTE(andreykurilin): the info method of platforms allows
+                    #   to return whatever single platform wants, i.e
+                    #   Platform X can return just a version and no services
+                    #   at all. Checking for 'services' key there is not a
+                    #   solution, since the value of it can have the format
+                    #   which differs from openstack-like (the old design of
+                    #   Deployment component)
+                    result[name] = [{"services": []}]
 
         return result
 
@@ -398,7 +333,7 @@ class _Task(APIGroup):
                 uuid=deployment["uuid"],
                 status=deployment["status"])
 
-        return objects.Task(deployment_uuid=deployment["uuid"],
+        return objects.Task(env_uuid=deployment["uuid"],
                             tags=tags).to_dict()
 
     def validate(self, deployment, config, task_instance=None, task=None):
@@ -421,7 +356,7 @@ class _Task(APIGroup):
             task = objects.Task.get(task)
             deployment = task["deployment_uuid"]
         else:
-            task = objects.Task(deployment_uuid=deployment, temporary=True)
+            task = objects.Task(env_uuid=deployment, temporary=True)
         deployment = objects.Deployment.get(deployment)
 
         try:
@@ -431,7 +366,7 @@ class _Task(APIGroup):
                 LOG.exception("Invalid Task")
             raise exceptions.InvalidTaskException(str(e))
 
-        engine.TaskEngine(config, task, deployment).validate()
+        engine.TaskEngine(config, task, deployment.env_obj).validate()
 
     def start(self, deployment, config, task=None, abort_on_sla_failure=False):
         """Validate and start a task.
@@ -481,7 +416,7 @@ class _Task(APIGroup):
                                 description=config.description)
 
         task_engine = engine.TaskEngine(
-            config, task, deployment,
+            config, task, deployment.env_obj,
             abort_on_sla_failure=abort_on_sla_failure)
 
         task_engine.validate()
@@ -490,11 +425,7 @@ class _Task(APIGroup):
         LOG.info("Run Task %s against Deployment %s"
                  % (task["uuid"], deployment["uuid"]))
 
-        try:
-            task_engine.run()
-        except Exception:
-            deployment.update_status(consts.DeployStatus.DEPLOY_INCONSISTENT)
-            raise
+        task_engine.run()
 
         return task["uuid"], task.get_status(task["uuid"])
 
@@ -538,9 +469,10 @@ class _Task(APIGroup):
         :param task_uuid: The UUID of the task
         :param force: If set to True, then delete the task despite to the
                       status
-        :raises TaskInvalidStatus: when the status of the task is not
-                                   in FINISHED, FAILED or ABORTED and
-                                   the force argument is not True
+        :raises DBConflict: when the status of the task is not
+                            in FINISHED, FAILED or ABORTED and
+                            the force argument is not True
+        :raises DBRecordNotFound: when task doesn't exist
         """
         if force:
             objects.Task.delete_by_uuid(task_uuid, status=None)
@@ -562,7 +494,7 @@ class _Task(APIGroup):
                 uuid=deployment["uuid"],
                 status=deployment["status"])
 
-        task_inst = objects.Task(deployment_uuid=deployment["uuid"],
+        task_inst = objects.Task(env_uuid=deployment["uuid"],
                                  tags=tags)
         task_inst.update_status(consts.TaskStatus.RUNNING)
         for subtask in task_results["subtasks"]:
@@ -578,7 +510,7 @@ class _Task(APIGroup):
                     name=workload["name"], description=workload["description"],
                     position=workload["position"], runner=workload["runner"],
                     runner_type=workload["runner_type"],
-                    context=workload["context"], hooks=workload["hooks"],
+                    contexts=workload["contexts"], hooks=workload["hooks"],
                     sla=workload["sla"], args=workload["args"])
 
                 chunk_size = CONF.raw_result_chunk_size
@@ -598,7 +530,8 @@ class _Task(APIGroup):
                     hooks_results=workload["hooks"],
                     start_time=workload["start_time"],
                     full_duration=workload["full_duration"],
-                    load_duration=workload["load_duration"])
+                    load_duration=workload["load_duration"],
+                    contexts_results=workload["contexts_results"])
             subtask_obj.update_status(consts.SubtaskStatus.FINISHED)
         task_inst.update_status(consts.SubtaskStatus.FINISHED)
 
@@ -606,17 +539,21 @@ class _Task(APIGroup):
 
         return task_inst.to_dict()
 
-    def export(self, tasks_uuids, output_type, output_dest=None):
+    def export(self, tasks, output_type, output_dest=None):
         """Generate a report for a task or a few tasks.
 
-        :param tasks_uuids: List of tasks UUIDs
+        :param tasks: List of tasks UUIDs or tasks results
         :param output_type: Plugin name of task exporter
         :param output_dest: Destination for task report
         """
 
         tasks_results = []
-        for task_uuid in tasks_uuids:
-            tasks_results.append(self.get(task_id=task_uuid, detailed=True))
+        tasks = tasks or []
+        for task in tasks:
+            if isinstance(task, dict):
+                tasks_results.append(task)
+            else:
+                tasks_results.append(self.get(task_id=task, detailed=True))
 
         errors = texporter.TaskExporter.validate(
             output_type, context={}, config={},
@@ -630,7 +567,8 @@ class _Task(APIGroup):
         reporter_cls = texporter.TaskExporter.get(output_type)
 
         LOG.info("Building '%s' report for the following task(s): '%s'."
-                 % (output_type, "', '".join(tasks_uuids)))
+                 % (output_type,
+                    "', '".join([task["uuid"] for task in tasks_results])))
         result = texporter.TaskExporter.make(reporter_cls,
                                              tasks_results,
                                              output_dest,
@@ -675,7 +613,7 @@ class _Verifier(APIGroup):
 
         try:
             verifier = self._get(name)
-        except exceptions.ResourceNotFound:
+        except exceptions.DBRecordNotFound:
             verifier = objects.Verifier.create(
                 name=name, source=source, system_wide=system_wide,
                 version=version, vtype=vtype, platform=platform,
@@ -762,7 +700,7 @@ class _Verifier(APIGroup):
         if deployment_id:
             LOG.info("Deleting deployment-specific data for verifier %s."
                      % verifier)
-            verifier.set_deployment(deployment_id)
+            verifier.set_env(deployment_id)
             verifier.manager.uninstall()
             LOG.info("Deployment-specific data has been successfully deleted!")
         else:
@@ -871,7 +809,7 @@ class _Verifier(APIGroup):
         """
         if not isinstance(verifier, objects.Verifier):
             verifier = self._get(verifier)
-        verifier.set_deployment(deployment_id)
+        verifier.set_env(deployment_id)
         LOG.info("Configuring verifier %s for deployment '%s' (UUID=%s)."
                  % (verifier,
                     verifier.deployment["name"],
@@ -932,7 +870,7 @@ class _Verifier(APIGroup):
                     verifier, verifier.status, consts.VerifierStatus.INSTALLED)
             )
 
-        verifier.set_deployment(deployment_id)
+        verifier.set_env(deployment_id)
         LOG.info("Overriding configuration of verifier %s for deployment '%s' "
                  "(UUID=%s)."
                  % (verifier,
@@ -1061,7 +999,7 @@ class _Verification(APIGroup):
                     verifier, verifier.status, consts.VerifierStatus.INSTALLED)
             )
 
-        verifier.set_deployment(deployment_id)
+        verifier.set_env(deployment_id)
         if not verifier.manager.is_configured():
             self.api.verifier.configure(verifier=verifier,
                                         deployment_id=deployment_id)
@@ -1212,7 +1150,7 @@ class _Verification(APIGroup):
         # [1] https://blueprints.launchpad.net/rally/+spec/verification-import
 
         verifier = self.api.verifier._get(verifier_id)
-        verifier.set_deployment(deployment_id)
+        verifier.set_env(deployment_id)
         LOG.info("Importing test results into a new verification for "
                  "deployment '%s' (UUID=%s), using verifier %s."
                  % (verifier.deployment["name"],
@@ -1267,6 +1205,7 @@ class API(object):
                  project="rally",
                  version=rally_version.version_string(),
                  default_config_files=config_files)
+            CONF.set_default("use_stderr", True)
 
             logging.setup("rally")
             if not CONF.get("log_config_append"):
