@@ -13,6 +13,7 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import inspect
 import os
 import re
 import six
@@ -27,12 +28,35 @@ from rally.common import yamlutils as yaml
 from rally import consts
 from rally import exceptions
 from rally.plugins.common import validators
+from rally.plugins.openstack.context.keystone import roles
 from rally.plugins.openstack.context.nova import flavors as flavors_ctx
 from rally.plugins.openstack import types as openstack_types
 
 LOG = logging.getLogger(__name__)
 
 
+def with_roles_ctx():
+    """Add roles to users for validate
+
+    """
+    def decorator(func):
+        def wrapper(*args, **kw):
+            func_type = inspect.getcallargs(func, *args, **kw)
+            config = func_type.get("config", {})
+            context = func_type.get("context", {})
+            if config.get("contexts", {}).get("roles") \
+                    and context.get("admin", {}):
+                context["config"] = config["contexts"]
+                rolegenerator = roles.RoleGenerator(context)
+                with rolegenerator:
+                    rolegenerator.setup()
+                    func(*args, **kw)
+            else:
+                func(*args, **kw)
+        return wrapper
+    return decorator
+
+
 @validation.add("required_platform", platform="openstack", users=True)
 @validation.configure(name="image_exists", platform="openstack")
 class ImageExistsValidator(validation.Validator):
@@ -48,6 +72,7 @@ class ImageExistsValidator(validation.Validator):
         self.param_name = param_name
         self.nullable = nullable
 
+    @with_roles_ctx()
     def validate(self, context, config, plugin_cls, plugin_cfg):
 
         image_args = config.get("args", {}).get(self.param_name)
@@ -71,10 +96,10 @@ class ImageExistsValidator(validation.Validator):
                 return
         try:
             for user in context["users"]:
-                clients = user["credential"].clients()
-                image_id = openstack_types.GlanceImage.transform(
-                    clients=clients, resource_config=image_args)
-                clients.glance().images.get(image_id)
+                image_processor = openstack_types.GlanceImage(
+                    context={"admin": {"credential": user["credential"]}})
+                image_id = image_processor.pre_process(image_args, config={})
+                user["credential"].clients().glance().images.get(image_id)
         except (glance_exc.HTTPNotFound, exceptions.InvalidScenarioArgument):
             self.fail("Image '%s' not found" % image_args)
 
@@ -91,6 +116,7 @@ class ExternalNetworkExistsValidator(validation.Validator):
         super(ExternalNetworkExistsValidator, self).__init__()
         self.param_name = param_name
 
+    @with_roles_ctx()
     def validate(self, context, config, plugin_cls, plugin_cfg):
 
         ext_network = config.get("args", {}).get(self.param_name)
@@ -138,6 +164,7 @@ class RequiredNeutronExtensionsValidator(validation.Validator):
             self.req_ext = [extensions]
             self.req_ext.extend(args)
 
+    @with_roles_ctx()
     def validate(self, context, config, plugin_cls, plugin_cfg):
         clients = context["users"][0]["credential"].clients()
         extensions = clients.neutron().list_extensions()["extensions"]
@@ -178,8 +205,9 @@ class FlavorExistsValidator(validation.Validator):
         if not flavor_value:
             self.fail("Parameter %s is not specified." % param_name)
         try:
-            flavor_id = openstack_types.Flavor.transform(
-                clients=clients, resource_config=flavor_value)
+            flavor_processor = openstack_types.Flavor(
+                context={"admin": {"credential": clients.credential}})
+            flavor_id = flavor_processor.pre_process(flavor_value, config={})
             flavor = clients.nova().flavors.get(flavor=flavor_id)
             return flavor
         except (nova_exc.NotFound, exceptions.InvalidScenarioArgument):
@@ -189,6 +217,7 @@ class FlavorExistsValidator(validation.Validator):
                 pass
             self.fail("Flavor '%s' not found" % flavor_value)
 
+    @with_roles_ctx()
     def validate(self, context, config, plugin_cls, plugin_cfg):
         # flavors do not depend on user or tenant, so checking for one user
         # should be enough
@@ -245,8 +274,9 @@ class ImageValidOnFlavorValidator(FlavorExistsValidator):
                 }
                 return image
         try:
-            image_id = openstack_types.GlanceImage.transform(
-                clients=clients, resource_config=image_args)
+            image_processor = openstack_types.GlanceImage(
+                context={"admin": {"credential": clients.credential}})
+            image_id = image_processor.pre_process(image_args, config={})
             image = clients.glance().images.get(image_id)
             if hasattr(image, "to_dict"):
                 # NOTE(stpierre): Glance v1 images are objects that can be
@@ -263,6 +293,7 @@ class ImageValidOnFlavorValidator(FlavorExistsValidator):
         except (glance_exc.HTTPNotFound, exceptions.InvalidScenarioArgument):
             self.fail("Image '%s' not found" % image_args)
 
+    @with_roles_ctx()
     def validate(self, context, config, plugin_cls, plugin_cfg):
 
         flavor = None
@@ -423,6 +454,7 @@ class ValidateHeatTemplateValidator(validation.Validator):
             self.params = [params]
             self.params.extend(args)
 
+    @with_roles_ctx()
     def validate(self, context, config, plugin_cls, plugin_cfg):
 
         for param_name in self.params:
@@ -462,6 +494,7 @@ class RequiredCinderServicesValidator(validation.Validator):
         super(RequiredCinderServicesValidator, self).__init__()
         self.services = services
 
+    @with_roles_ctx()
     def validate(self, context, config, plugin_cls, plugin_cfg):
 
         clients = context["admin"]["credential"].clients()
@@ -537,6 +570,7 @@ class VolumeTypeExistsValidator(validation.Validator):
         self.param = param_name
         self.nullable = nullable
 
+    @with_roles_ctx()
     def validate(self, context, config, plugin_cls, plugin_cfg):
         volume_type = config.get("args", {}).get(self.param, False)
 
